@@ -199,6 +199,18 @@ VM::EC2::Volume->size.
 Refreshes the snapshot from information provided by AWS. Use before
 checking progress or other changeable elements.
 
+=head2 $snapshot_copy = $snapshot->copy(-region=>$dest_region, -description=>$desc)
+
+Copies the snapshot to the same or different region.
+
+Required Argument:
+ -region        The region to copy the snapshot to
+
+Optional Argument:
+ -description   Description of the new snapshot
+
+Returns a VM::EC2::Snapshot object if successful.
+
 =head1 STRING OVERLOADING
 
 When used in a string context, this object will interpolate the
@@ -263,6 +275,7 @@ sub to_volumes {
 
 sub refresh {
     my $self = shift;
+    local $self->aws->{raise_error} = 1;
     my $s = $self->aws->describe_snapshots($self);
     %$self  = %$s;
 }
@@ -356,4 +369,19 @@ sub product_codes {
     return map {VM::EC2::ProductCode->new($_,$self->aws)} @codes;
 }
 
+sub copy {
+    my $self = shift;
+    my %args = @_;
+    my $snap_id = $self->snapshotId;   
+    my $desc    = $args{-description} || $args{-desc};
+    my $region  = $args{-region} or croak "copy(): -region argument required";
+    my $orig_region = $self->aws->region;
+
+    # create a new EC2 object for the destination
+    my $dest_aws = $self->aws->clone;
+    $dest_aws->region($region);
+    my $snapshot = $dest_aws->copy_snapshot(-source_region=>$orig_region, -source_snapshot_id=>$snap_id, -description=>$desc);
+    return $snapshot;
+}
+
 1;
